@@ -50,11 +50,25 @@ pub enum Card {
     SetGlobalVar(VarNode),
     SetVar(VarNode),
     ReadVar(VarNode),
-    Repeat(LaneNode),
-    While(LaneNode),
+    Repeat {
+        /// Loop variable is written into this variable
+        i: Option<VarNode>,
+        body: Box<Card>,
+    },
+    While {
+        condition: Box<Card>,
+        body: Box<Card>,
+    },
     ForEach {
-        variable: VarNode,
-        lane: LaneNode,
+        /// Loop variable is written into this variable
+        i: Option<VarNode>,
+        /// The key is written into this variable
+        k: Option<VarNode>,
+        /// The object is written into this variable
+        o: Option<VarNode>,
+        /// Variable that is iterated on
+        variable: Box<Card>,
+        body: Box<Card>,
     },
     /// Single card that decomposes into multiple cards
     CompositeCard(Box<CompositeCard>),
@@ -94,8 +108,8 @@ impl Card {
             Card::ClearStack => "ClearStack",
             Card::ScalarNil => "ScalarNil",
             Card::Return => "Return",
-            Card::Repeat(_) => "Repeat",
-            Card::While(_) => "While",
+            Card::Repeat { .. } => "Repeat",
+            Card::While { .. } => "While",
             Card::IfElse { .. } => "IfElse",
             Card::GetProperty => "GetProperty",
             Card::SetProperty => "SetProperty",
@@ -111,8 +125,8 @@ impl Card {
             Card::IfElse { .. }
             | Card::ReadVar(_)
             | Card::SetVar(_)
-            | Card::While(_)
-            | Card::Repeat(_)
+            | Card::While { .. }
+            | Card::Repeat { .. }
             | Card::ForEach { .. }
             | Card::Pass
             | Card::CompositeCard { .. } => None,
@@ -190,8 +204,6 @@ impl Card {
             | Instruction::Goto
             | Instruction::GotoIfTrue
             | Instruction::GotoIfFalse
-            | Instruction::Repeat
-            | Instruction::BeginRepeat
             | Instruction::ForEach
             | Instruction::BeginForEach => {}
         };
